using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace CSSample
{

	struct BITMAPINFOHEADER 
	{
		public UInt32	biSize; 
		public Int32	biWidth; 
		public Int32	biHeight; 
		public UInt16	biPlanes; 
		public UInt16	biBitCount; 
		public UInt32	biCompression; 
		public UInt32	biSizeImage; 
		public Int32	biXPelsPerMeter; 
		public Int32	biYPelsPerMeter; 
		public UInt32	biClrUsed; 
		public UInt32	biClrImportant; 
	};

	struct BITMAPFILEHEADER
	{
		public UInt16	bfType;
		public UInt32	bfSize;
		public UInt16	bfReserved1;
		public UInt16	bfReserved2;
		public UInt32	bfOffBits; 
	};

	struct RGBQUAD 
	{
		public Byte    rgbBlue; 
		public Byte    rgbGreen; 
		public Byte    rgbRed; 
		public Byte    rgbReserved; 
	};
	
	public class BitmapFile
	{
		bool			m_isOpen;
		int				m_currentFileOffset;
		int				m_totalImageBytesWritten;
		bool			m_dataWritten;
		BinaryWriter	m_binaryWriter;

		BITMAPFILEHEADER	m_bmFileHeader;
		BITMAPINFOHEADER	m_bmInfoHeader;
		RGBQUAD[]			m_colorTable;
		const int m_sizeOfBitmapFileHeader = 14;
		const int m_sizeOfBitmapInfoHeader = 40;

		/* constants for the biCompression field of BITMAPINFOHEADER*/
		const UInt32 BI_RGB =		0;
		const UInt32 BI_RLE8 =		1;
		const UInt32 BI_RLE4 =		2;
		const UInt32 BI_BITFIELDS =	3;
		const UInt32 BI_JPEG =		4;
		const UInt32 BI_PNG =		5;


		public BitmapFile()
		{
			m_isOpen = false;
			m_totalImageBytesWritten = 0;
			m_dataWritten = false;
			m_currentFileOffset = 0;

			m_bmFileHeader.bfType = 0x4D42;		// the bytes "BM" for bitmap
			m_bmFileHeader.bfReserved1 = 0;
			m_bmFileHeader.bfReserved2 = 0;

			m_bmInfoHeader.biSize = m_sizeOfBitmapInfoHeader;
			m_bmInfoHeader.biPlanes = 1;
			m_bmInfoHeader.biCompression = BI_RGB;
			m_bmInfoHeader.biXPelsPerMeter = 0;
			m_bmInfoHeader.biYPelsPerMeter = 0;
			m_bmInfoHeader.biClrUsed = 0;
			m_bmInfoHeader.biClrImportant = 0; 

			// initialize color table
			m_colorTable = new RGBQUAD[256];	// allocate enough for grayscale
			for (int i = 0; i < 256; i++)
			{
				m_colorTable[i].rgbBlue =
				m_colorTable[i].rgbGreen =
				m_colorTable[i].rgbRed = (Byte)i;
				m_colorTable[i].rgbReserved = 0;
			}
		}

		public void Open(string filepath)
		{
			// If a file is currently open, close it.
			if (m_isOpen)
			{
				m_binaryWriter.Close();
				m_isOpen = false;
			}

			m_binaryWriter = new BinaryWriter(
				new FileStream(filepath, FileMode.Create, FileAccess.Write));

			m_dataWritten = false;
			m_currentFileOffset = 0;
			m_totalImageBytesWritten = 0;
			m_isOpen = true;

			return;

		}

		public bool IsOpen()
		{
			return m_isOpen;
		}

		public void Close()
		{

			if (!m_isOpen)
			{
				return;
			}

			// Update the total file size according to the actual number of bytes written.
			m_bmFileHeader.bfSize = (uint)(m_bmFileHeader.bfOffBits + m_totalImageBytesWritten);
			m_bmInfoHeader.biSizeImage = (uint)m_totalImageBytesWritten;

			Seek(0);
			WriteHeaders();

			m_binaryWriter.Close();
			m_isOpen = false;

		}

		public bool SetWidth(int width)
		{
			m_bmInfoHeader.biWidth = width;
			return true;
		}

		public bool SetHeight(int height)
		{
			m_bmInfoHeader.biHeight = height;
			return true;
		}

		public bool SetBitsPerPixel(ushort bitsPerPixel)
		{
			if((bitsPerPixel == 1) || (bitsPerPixel == 8) || (bitsPerPixel == 24))
			{
				m_bmInfoHeader.biBitCount = bitsPerPixel;
				return true;
			}
			return false;
		}

		public void SetResolution(int dpi)
		{
			// convert to dots per meter
			const float INCHES_PER_METER = 39.3700787f;

			m_bmInfoHeader.biXPelsPerMeter = (int)((dpi * INCHES_PER_METER) + 0.5f);
			m_bmInfoHeader.biYPelsPerMeter = m_bmInfoHeader.biXPelsPerMeter;
		}


		public bool Seek(int filePosition)
		{
			if(m_isOpen)
			{
				try
				{
					m_binaryWriter.Seek(filePosition, SeekOrigin.Begin);
				}
				catch
				{
					return false;
				}
			}
			else
			{
				return false;
			}
			return true;
		}

		private byte[] StructureToByteArray(object aStruct)
		{
			byte[] buff = new byte[Marshal.SizeOf(aStruct.GetType())];//Create Buffer
			GCHandle handle = GCHandle.Alloc(buff, GCHandleType.Pinned);//Hands off GC
			//Marshal the structure
			Marshal.StructureToPtr(aStruct, handle.AddrOfPinnedObject(), false);
			handle.Free();
			return buff;
		}

		// Although a little uglier, using BinaryWriter is faster than writing out a 
		// whole structure as a block when the structure is less than about 40 fields.
		void WriteHeaders()
		{
			m_binaryWriter.Write(m_bmFileHeader.bfType);
			m_binaryWriter.Write(m_bmFileHeader.bfSize);
			m_binaryWriter.Write(m_bmFileHeader.bfReserved1);
			m_binaryWriter.Write(m_bmFileHeader.bfReserved2);
			m_binaryWriter.Write(m_bmFileHeader.bfOffBits);

			m_binaryWriter.Write(m_bmInfoHeader.biSize);
			m_binaryWriter.Write(m_bmInfoHeader.biWidth);
			m_binaryWriter.Write(m_bmInfoHeader.biHeight);
			m_binaryWriter.Write(m_bmInfoHeader.biPlanes);
			m_binaryWriter.Write(m_bmInfoHeader.biBitCount);
			m_binaryWriter.Write(m_bmInfoHeader.biCompression);
			m_binaryWriter.Write(m_bmInfoHeader.biSizeImage);
			m_binaryWriter.Write(m_bmInfoHeader.biXPelsPerMeter);
			m_binaryWriter.Write(m_bmInfoHeader.biYPelsPerMeter);
			m_binaryWriter.Write(m_bmInfoHeader.biClrUsed);
			m_binaryWriter.Write(m_bmInfoHeader.biClrImportant);
		}


		// When Write() is called for the first time and image data is written 
		// for the first time, we commit to the scan parameters as they are set 
		// so that we write out the appropriate color table. The color table 
		// needs to go before the image data.
		private void DoFirstWrite()
		{
			const int RGBQUADSIZE = 4;

			// With gray or black & white we need to write out a color table.
			int colorTableEntriesToWrite = 0;
			int colorTableSizeInBytes = 0;
			if (m_bmInfoHeader.biBitCount == 8)	// grayscale
			{
				m_colorTable[1].rgbBlue =
				m_colorTable[1].rgbGreen =
				m_colorTable[1].rgbRed = 1;
				colorTableSizeInBytes = RGBQUADSIZE * 256;
				colorTableEntriesToWrite = 256;

			}
			else if (m_bmInfoHeader.biBitCount == 1)	// B&W
			{
				m_colorTable[1].rgbBlue =
				m_colorTable[1].rgbGreen =
				m_colorTable[1].rgbRed = 0xFF;
				colorTableSizeInBytes = RGBQUADSIZE * 2;
				colorTableEntriesToWrite = 2;

			}

			// Store total header size for use at the end of the scan
			m_bmFileHeader.bfOffBits =	(uint)(m_sizeOfBitmapFileHeader + 
												m_sizeOfBitmapInfoHeader + 
												colorTableSizeInBytes);

			Seek(0);

			try
			{
				WriteHeaders();
			}
			catch
			{
				return ;
			}

			// write out the color table if necessary
			for (int i = 0; i < colorTableEntriesToWrite; i++)
			{
				m_binaryWriter.Write(m_colorTable[i].rgbBlue);
				m_binaryWriter.Write(m_colorTable[i].rgbGreen);
				m_binaryWriter.Write(m_colorTable[i].rgbRed);
				m_binaryWriter.Write(m_colorTable[i].rgbReserved);
			}

			m_currentFileOffset = (int)m_bmFileHeader.bfOffBits;
		}

		// Write a buffer out to the file.
		public bool Write(byte[] buffer, uint lengthInBytes)
		{
			bool result = false;
	
			if(!m_isOpen)
			{
				return result;
			}

			if(m_dataWritten == false)
			{
				DoFirstWrite();
				m_dataWritten = true;
			}

			try
			{
				m_binaryWriter.BaseStream.Write(buffer, 0, (int)lengthInBytes);
			}
			catch
			{
				return false;
			}

			m_totalImageBytesWritten += (int)lengthInBytes;
			result = true;

			return result;
		}

	}
}
