/* SDKSample
//
//	Performs a 4-inch long color scan at 100 DPI and saves the resulting image
//	to a file named "sample.tif".
*/

#include <assert.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <windows.h>
#include "TiffDump.h"
#include "ScannerAPI.h"

int SetScanProperties(int32 scanMode);
int CheckForCalibration();


int main(int argc, char* argv[])
{
	SIResult		result;
	CTiffDump		tiff;
	int				tiffResult;
	int32			scanWidthInPixels;
	int32			scanWidthInBytes;
	int32			xResolution;
	uint8			*pLineBuffer = NULL;
	int32			linesRemaining = 0;
	int32			linesToRead  = 0;
	uint32			linesReturned = 0;
	int				ch;
	SIPaperStatus	paperStatus;
	SIProperty		prop;
	int32			scanMode;
	int				i;		// loop variable
	char*			pScannerName;

	/* Open the scanner interface */
#if defined(DPORT_487)
	pScannerName = "DocketPORT487";
#elif defined(DPORT_488)
	pScannerName = "DocketPORT488";
#elif defined(DPORT_467)
	pScannerName = "DocketPORT467";
#elif defined(DPORT_468)
	pScannerName = "DocketPORT468";
#elif defined(DPORT_667)
	pScannerName = "DocketPORT667";
#elif defined(DPORT_687)
	pScannerName = "DocketPORT687";
#else
	#error scanner model not defined.
#endif

	result = SI_OpenInterface(pScannerName);

	if(result != SIR_SUCCESS)
	{
		printf("Error opening scanner. Hit any key to exit.\n");
		/* Wait for a keystroke */
		while(!_kbhit())
		{};
		return 1;
	}

	scanMode = SI_SCANMODE_RGB;
	// Check for a command line parameter that specifies the scan mode.
	// Valid arguments are:   
	//				/bw  
	//				/gray
	//				/color
	// If there are no parameters, the default is color.
	// If there are multiple parameters, the last one takes precedence.
 	for(i = 1; i < argc; i++)
	{
		if(argv[i][0] == '/')
		{
			if(0 == (strcmp(&argv[i][1], "bw")))
			{
				scanMode = SI_SCANMODE_BW;
			}
			else if(0 == (strcmp(&argv[i][1], "gray")))
			{
				scanMode = SI_SCANMODE_GRAY;
			}
			else if(0 == (strcmp(&argv[i][1], "color")))
			{
				scanMode = SI_SCANMODE_RGB;
			}
		}
	}

	/* Make sure the scanner is calibrated. */
	CheckForCalibration();


	/* Make sure there is paper in the scanner. */
	paperStatus = SI_PS_PAPER_OUT;
	while(paperStatus == SI_PS_PAPER_OUT)
	{
		printf("Insert paper and press any key to begin the scan.\n");

		/* Wait for a keystroke */
		while(!_kbhit())
		{};

		ch = _getch();	/* pull in the character. */

		/* Check if the paper is in. */
		result = SI_GetPaperStatus(&paperStatus);

		if(result != SIR_SUCCESS)
		{
			printf("Error getting paper status.\n");
		}
	}


	/*	Setup the scan properties. */
	SetScanProperties(scanMode);

	/*	Collect information from the scanner.
		We could have saved this information as we set it up in SetScannerProperties,
		but instead we'll retrieve it again. */

	prop.propertyID = SIP_SCAN_WIDTH_IN_PIXELS;
	result = SI_GetProperty(&prop);
	assert((result == SIR_SUCCESS) && (prop.containerType== SICON_RANGE) && (prop.itemType == SI_INT32));
	scanWidthInPixels = prop.range.current.iVal;

	prop.propertyID = SIP_LINE_WIDTH_IN_BYTES;
	result = SI_GetProperty(&prop);
	assert(result == SIR_SUCCESS);
	scanWidthInBytes = prop.single.current.iVal;

	prop.propertyID = SIP_XRESOLUTION;
	result = SI_GetProperty(&prop);
	assert(result == SIR_SUCCESS);
	xResolution = prop.list.current.iVal;

	/* Create output file. */
	tiffResult = tiff.Create("sample.tif");
	if (tiffResult != CTiffDump::kTDNoErr)
	{
		printf("Error creating output file.\n");
		SI_CloseInterface();
		return 1;
	}
	tiff.SetWidthInPixels(scanWidthInPixels);
	tiff.SetNumChannels((scanMode == SI_SCANMODE_RGB) ? 3 : 1);	  /* 3 for color, 1 for B&W or gray */
	tiff.SetBitsPerChannel((scanMode == SI_SCANMODE_BW) ? 1 : 8); /* 1 for B&W, 8 for color or gray */
	assert(uint32(tiff.GetRowBytes()) == scanWidthInBytes);
	tiff.SetResolution(xResolution);

	/* Allocate memory for 10 lines.*/
	pLineBuffer = (uint8*)malloc(scanWidthInBytes * 10);
	assert(pLineBuffer);

	/* inquire number of lines to scan */
	prop.propertyID = SIP_SCAN_LENGTH_IN_LINES;
	result = SI_GetProperty(&prop);
	assert(result == SIR_SUCCESS);
	linesRemaining = prop.range.current.iVal;

	/* Start the scan */
	result = SI_StartScan();
	assert(result == SIR_SUCCESS);
	if(result != SIR_SUCCESS)
	{
		printf("Error starting the scan.\n");
		free(pLineBuffer);
		SI_CloseInterface();
		return 1;
	}

	/* Enter the loop to read image data from the scanner and save 
		it to the output file.
	*/
	while(linesRemaining > 0)
	{
		linesToRead = (linesRemaining < 10) ? linesRemaining : 10;

		linesReturned = 0;

		result = SI_ReadImageData(pLineBuffer, linesToRead, 0, &linesReturned  );


		if((result != SIR_SUCCESS) && (result != SIR_ENDOFDATA))
		{
			linesRemaining = 0;
			break;
		}

		if(result == SIR_ENDOFDATA)
		{
			printf("linesReturned = %d", linesReturned);
			linesRemaining = 0;
		}
		
		if(linesReturned > 0)
		{
			tiffResult = tiff.AppendLines(linesReturned, pLineBuffer);
			assert(tiffResult == CTiffDump::kTDNoErr);

			linesRemaining -= linesReturned;
		}
		else
		{
			Sleep(10);
		}

	};

	/* End the scan */
	SI_EndScan();

	/* Feed the paper out of the scanner. */
	SI_FeedPaperOut();

	free(pLineBuffer);

	tiff.Cleanup();


	SI_CloseInterface();

	return 0;
}


int CheckForCalibration()
{
	int ch;
	SIPaperStatus paperStatus;
	SICalibrationState calState;
	SIResult result;

	result = SI_IsCalibrated(&calState);
	
	if(result == SIR_INTERFACE_NOT_OPEN)
	{
		return 1;
	}

	if(result == SIR_SUCCESS)
	{
		if(calState == SI_TRUE)
		{
			/* Scanner is calibrated. So just return. */
			return 0;
		}
	}

	// 
	printf("The scanner is not calibrated. Do you want to calibrate it?\n");
	printf("Hit 'y' to calibrate, or any other key to skip calibration.\n\n");

	/* Wait for a keystroke */
	while(!_kbhit())
	{};

	ch = _getch();
	ch = toupper( ch );
	if(ch != 'Y')
	{
		/* skip calibration. */
		return 0;
	}

	paperStatus = SI_PS_PAPER_OUT;
	while(paperStatus == SI_PS_PAPER_OUT)
	{
		printf("Insert the calibration target and press any key to start calibration.\n");

		/* Wait for a kestroke */
		while(!_kbhit())
		{};

		ch = _getch();	/* pull in the character. */

		/* Check if the target is in. */
		result = SI_GetPaperStatus(&paperStatus);

		if(result != SIR_SUCCESS)
		{
			printf("Error getting paper status.\n");
			return 1;
		}
	}

	printf("Calibrating....\n");

	/* Perform the calibration. */
	result = SI_Calibrate(SI_CT_DEFAULT, NULL);
    
	if(result == SIR_SUCCESS)
	{
		printf("Calibration completed successfully.\n");
	}
	else
	{
		printf("There was an error during calibration.\n");
	}

	return (result == SIR_SUCCESS) ? 0 : 1;
}


int SetScanProperties(int32 scanMode)
{
	SIResult	result;
	SIProperty	prop;
	int32		found;
	int32		resItem;
	int32		xResolution;


	/* Set scan resolution */
	/* In this first example, we retrieve the property to confirm that the 
		container type and item type are what we expect. Often you don't 
		need to do this if you're sure of the container and item type for
		the scanner model. But if you're developing code for more than one 
		scanner, you may want to check.

		To get a property, you need to set only the propertyID of the
		structure and then pass that to SI_GetProperty().
	*/
	prop.propertyID = SIP_XRESOLUTION;
	result = SI_GetProperty(&prop);
	
	assert(result == SIR_SUCCESS);
	assert((prop.containerType == SICON_LIST) && (prop.itemType == SI_INT32));

	/* As an example, let's check to make sure 100 DPI is a valid resolution */
	found = 0;
	for(resItem = 0; resItem < prop.list.numItems; resItem++)
	{
		if(prop.list.items.piVal[resItem] == 100)
		{
			/* Yes, 100 DPI is one of the supported resolutions for this scanner. */
			found = 1;
			break;
		}
	}

	/*	Using the SICON_LIST container, set the X resolution.
		If 100 DPI was valid, use that, otherwise use the lowest 
		available resolution.	
		To set the current value, you need to make sure four fields 
		are filled in: propertyID, containerType, itemType and the current
		value within the appropriate range. The first three of those fields 
		should already be set as a result to the call to SI_GetProperty.
		So just set the current value of the list container. 
		If we had not called SI_GetProperty, we'd need to set all four.*/
	xResolution = found ? 100 : prop.list.items.piVal[0];
	prop.list.current.iVal = xResolution;

	result = SI_SetProperty(&prop);
	assert(result == SIR_SUCCESS);


	/*	For most scanners, X and Y resolution must be the same. So the
		Y resolution property will have changed to match the X resolution 
		we just set. As an example, get the Y resolution to check this. */
	prop.propertyID = SIP_YRESOLUTION;
	result = SI_GetProperty(&prop);
	assert(result == SIR_SUCCESS);
	if(prop.list.current.iVal == xResolution)
	{
		/* Y resolution same as X resolution */
	}
	else
	{
		/* Y resolution independent of X resolution */
	}


	/* Now set other scan properties */

	/* Set the Y offset to 0.
		It's a good idea to set X and Y offsets before setting width and length
		because the offset's value can affect the maximum valid width or length.
	*/
	prop.propertyID = SIP_YOFFSET;
	prop.containerType = SICON_SINGLE;
	prop.itemType = SI_INT32;
	prop.single.current.iVal = 0;
	result = SI_SetProperty(&prop);
	assert(result == SIR_SUCCESS);
	
	/* Set the X offset to 0 */
	prop.propertyID = SIP_XOFFSET;
	prop.containerType = SICON_SINGLE;
	prop.itemType = SI_INT32;
	prop.single.current.iVal = 0;
	result = SI_SetProperty(&prop);
	assert(result == SIR_SUCCESS);


	/* Scan Width. Set the width to the maximum allowed so we get full width. */
	prop.propertyID = SIP_SCAN_WIDTH_IN_PIXELS;
	result = SI_GetProperty(&prop);
	assert(result == SIR_SUCCESS);

	prop.range.current.iVal = prop.range.maximum.iVal;
	result = SI_SetProperty(&prop);
	assert(result == SIR_SUCCESS);
	

	/* Set scan length. We'll scan four inches at the current resolution. */
	prop.propertyID = SIP_SCAN_LENGTH_IN_LINES;
	prop.containerType = SICON_SINGLE;
	prop.itemType = SI_INT32;
	prop.single.current.iVal = 4 * xResolution;
	result = SI_SetProperty(&prop);
	assert(result == SIR_SUCCESS);


	/* Set scan mode. */
	prop.propertyID = SIP_SCAN_MODE;
	prop.containerType = SICON_SINGLE;
	prop.itemType = SI_INT32;
	prop.single.current.iVal = scanMode;	// Passed in as a parameter.
											// Valid values are: SI_SCANMODE_RGB, SI_SCANMODE_GRAY, or SI_SCANMODE_BW
	result = SI_SetProperty(&prop);
	assert(result == SIR_SUCCESS);

	return 0;
}

