Imports System.Diagnostics
Imports ScannerMng
Imports SIP = ScannerMng.SIProperty	' SIP will be shorthand for ScannerMng.SIProperty

Public Class VBSample

	' This is the filename of the DLL providing the "native" API.
	' For the sample, use the DocketPORT 487 DLL.
	Private m_ScannerDllFilename As String = "DPORT487.dll"

	' This is the scanner name that is passed to SI_OpenInterface 
	' to specify we want to open a DocketPORT 487.
	Private m_ScannerName As String = "DocketPORT487"

	' The filename of the image file to be written.
	Private m_outFilename As String = "VBSample.bmp"

	Private m_scanner As SIScanner
	Private m_scannerReady As Boolean
	Private m_progressDialog As ProgressDialog
	Private Const m_blackAndWhiteText As String = "B&W"
	Private Const m_grayText As String = "Gray"
	Private Const m_colorText As String = "Color"


	Public Sub New()

		' This call is required by the Windows Form Designer.
		InitializeComponent()

		' Add any initialization after the InitializeComponent() call.

		m_scannerReady = False

		Try
			m_scanner = New SIScanner(m_ScannerDllFilename)
		Catch e As ApplicationException
			MessageBox.Show(e.Message)
			Return
		End Try

		Dim result As SIResult = m_scanner.SI_OpenInterface(m_ScannerName)

		If result <> SIResult.SIR_SUCCESS Then
			Dim str As String = ""
			m_scanner.SI_GetLastErrorText(str)
			MessageBox.Show(str)
			Return
		End If

		m_scannerReady = True

		PopulateControls()

	End Sub
	' This function will query the scanner for its available scan modes 
	' and resolutions. Then it will add these to the UI controls. 
	' This is not strictly necessary. If you're only developing for one 
	' scanner model and you know the available modes and resolutions,
	' then you can just hard code them. But querying the scanner properties
	' is useful if you want to write code that works for more than one 
	' scanner model.
	'
	Private Sub PopulateControls()

		If Not m_scannerReady Then
			Return
		End If

		Dim prop As SIProperty = New SIProperty()
		Dim result As SIResult

		' Get Scan modes			
		prop.propertyID = SIP.SIP_SCAN_MODE
		result = m_scanner.SI_GetProperty(prop)
		Debug.Assert(result = SIResult.SIR_SUCCESS)
		Debug.Assert((prop.containerType = SIP.SICON_LIST) AndAlso (prop.itemType = SIP.SI_INT32))

		' Add the scan modes to the scan mode combo box
		scanModeComboBox.Items.Clear()
		For i As Integer = 0 To prop.list.numItems - 1
			If prop.list.items.piVal(i) = SIP.SI_SCANMODE_BW Then

				scanModeComboBox.Items.Add(m_blackAndWhiteText)

			ElseIf prop.list.items.piVal(i) = SIP.SI_SCANMODE_GRAY Then

				scanModeComboBox.Items.Add(m_grayText)

			ElseIf prop.list.items.piVal(i) = SIP.SI_SCANMODE_RGB Then

				scanModeComboBox.Items.Add(m_colorText)

			End If

			If prop.list.items.piVal(i) = prop.list.current.iVal Then
				scanModeComboBox.SelectedIndex = i
			End If
		Next

		' Although X resolution and Y resolution are separate properties,
		' scanners will typically have them locked together. X and Y will 
		' always be the same.
		prop.propertyID = SIP.SIP_XRESOLUTION
		result = m_scanner.SI_GetProperty(prop)
		Debug.Assert(result = SIResult.SIR_SUCCESS)
		Debug.Assert((prop.containerType = SIP.SICON_LIST) AndAlso (prop.itemType = SIP.SI_INT32))

		' Add the resolutions to the resolution combobox.
		resolutionComboBox.Items.Clear()
		For i As Integer = 0 To prop.list.numItems - 1
			resolutionComboBox.Items.Add(prop.list.items.piVal(i))
		Next

		resolutionComboBox.SelectedItem = prop.list.current.iVal

	End Sub

	Private Sub CleanUp(ByVal disposing As Boolean)

		If m_scannerReady AndAlso disposing Then
			m_scanner.SI_CloseInterface()
			m_scannerReady = False
		End If

	End Sub


	' This function is called by the driver during calibration to 
	' report progress and allow the user to cancel.
	' Because of the way calibration works, there can be a few seconds 
	' between calls, so the GUI response may not be immediate.
	Public Function CalibrationProgressHandler(ByVal percentComplete As UInt32) As SICallbackResult

		m_progressDialog.Percent = percentComplete

		' This will process any pending messages, redrawing the progress 
		' meter and also responding to presses of the Cancel button.
		Application.DoEvents()

		If m_progressDialog.Canceled Then
			' Cancel the calibration
			Return SICallbackResult.CANCEL
		End If

		Return SICallbackResult.CONTINUE

	End Function

	' Returns false if error.
	' Return true if:
	'	The scanner is already calibrated.
	'	The scanner performs the calibration.
	'	The user elected to skip calibration.
	'	(In this sample program, you can scan without calibrating.)
	Private Function CheckForCalibration() As Boolean

		Dim calState As SICalibrationState = SICalibrationState.CALIBRATED

		Dim result As SIResult

		result = m_scanner.SI_IsCalibrated(calState)

		If result = SIResult.SIR_SUCCESS Then

			If calState = SICalibrationState.NOT_CALIBRATED Then

				Dim dlgRes As DialogResult = MessageBox.Show( _
				 "The scanner is not calibrated." & vbCrLf & "Do you want to calibrate now?", _
				 "ScanToFile", _
				 MessageBoxButtons.YesNo, _
				 MessageBoxIcon.Exclamation)


				If dlgRes = DialogResult.Yes Then

					If Not ConfirmPaper("Please insert the calibration paper and press OK." & vbCrLf & _
					 "Press Cancel to skip calibration.") Then

						Return True
					End If

					m_progressDialog = New ProgressDialog()
					m_progressDialog.Text = "Calibrating..."
					m_progressDialog.Show()
					Application.DoEvents()	' Allows the modeless dialog to fully draw itself

					result = m_scanner.SI_Calibrate(SICalibrationTarget.SI_CT_DEFAULT, _
					 New ProgressCallbackDelegate(AddressOf CalibrationProgressHandler))

					m_progressDialog.Close()
					m_progressDialog = Nothing

					If result = SIResult.SIR_SUCCESS Then
						MessageBox.Show("Calibration was successful.", "ScanToFile")
					Else

						Dim errStr As String = ""
						m_scanner.SI_GetLastErrorText(errStr)
						MessageBox.Show(errStr, "ScanToFile", _
						 MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
						Return False
					End If
				End If
			End If
		Else
			Return False	' error
		End If

		Return True

	End Function


	' returns true when paper has been inserted and the user clicks OK.
	' returns false if the user wants to abort the scan.
	Private Function ConfirmPaper(ByVal message As String) As Boolean

		Dim paperStatus As SIPaperStatus = SIPaperStatus.SI_PS_PAPER_OUT

		Dim result As SIResult = m_scanner.SI_GetPaperStatus(paperStatus)
		Debug.Assert(result = SIResult.SIR_SUCCESS)

		While paperStatus = SIPaperStatus.SI_PS_PAPER_OUT

			Dim dlgRes As DialogResult = MessageBox.Show( _
			  message, _
			  "ScanToFile", _
			  MessageBoxButtons.OKCancel)

			If dlgRes = DialogResult.Cancel Then
				Return False	' skip 
			End If
			result = m_scanner.SI_GetPaperStatus(paperStatus)
			Debug.Assert(result = SIResult.SIR_SUCCESS)
		End While
		Return True
	End Function


	' This function retrives the current settings from the comboboxes and 
	' sets them to the scanner.
	Private Sub ControlsToProperties()

		Dim result As SIResult
		Dim prop As SIProperty = New SIProperty()

		'////////////////////////////////////////
		' Scan Mode
		'////////////////////////////////////////
		prop.propertyID = SIP.SIP_SCAN_MODE
		m_scanner.SI_GetProperty(prop)

		Dim str As String = scanModeComboBox.SelectedItem

		If str.Equals(m_blackAndWhiteText) Then

			prop.list.current.iVal = SIP.SI_SCANMODE_BW

		ElseIf str.Equals(m_grayText) Then

			prop.list.current.iVal = SIP.SI_SCANMODE_GRAY

		ElseIf str.Equals(m_colorText) Then

			prop.list.current.iVal = SIP.SI_SCANMODE_RGB

		End If

		result = m_scanner.SI_SetProperty(prop)
		Debug.Assert(result = SIResult.SIR_SUCCESS)

		'/////////////////////////////////////////
		'/ Resolution
		'/////////////////////////////////////////
		prop.propertyID = SIP.SIP_XRESOLUTION
		m_scanner.SI_GetProperty(prop)

		prop.list.current.iVal = resolutionComboBox.SelectedItem

		result = m_scanner.SI_SetProperty(prop)
		Debug.Assert(result = SIResult.SIR_SUCCESS)

		'///////////////////////////////////////////////////////////////
		' Scan width
		' To keep things simple, set the scan width to the default 
		' value for this resolution. Although, you can set any value 
		' between prop.range.minimum.iVal and prop.range.maximum.iVal.
		'///////////////////////////////////////////////////////////////
		prop.propertyID = SIP.SIP_SCAN_WIDTH_IN_PIXELS
		m_scanner.SI_GetProperty(prop)
		prop.range.current.iVal = prop.range.def.iVal

		result = m_scanner.SI_SetProperty(prop)
		Debug.Assert(result = SIResult.SIR_SUCCESS)

		'///////////////////////////////////////////////////////////////
		' Scan length
		' To keep things simple, set the scan length to the default 
		' value for this resolution. Although, you can set any value 
		' between prop.range.minimum.iVal and prop.range.maximum.iVal.
		'///////////////////////////////////////////////////////////////
		prop.propertyID = SIP.SIP_SCAN_LENGTH_IN_LINES
		m_scanner.SI_GetProperty(prop)
		prop.range.current.iVal = prop.range.def.iVal

		result = m_scanner.SI_SetProperty(prop)
		Debug.Assert(result = SIResult.SIR_SUCCESS)

		'//////////////////////////////////////////////////////////////
		' Channel Order
		'
		' The driver returns color data in the order R-G-B by default.
		' But Windows bitmap format requires B-G-R order.
		' So we need to specify B-G-R order from the scanner driver.
		'//////////////////////////////////////////////////////////////
		prop.propertyID = SIP.SIP_CHANNEL_ORDER
		m_scanner.SI_GetProperty(prop)
		Debug.Assert(result = SIResult.SIR_SUCCESS)
		Debug.Assert((prop.containerType = SIP.SICON_LIST) AndAlso (prop.itemType = SIP.SI_INT32))

		prop.list.current.iVal = SIP.SI_CO_BGR	' set the order to BGR

		result = m_scanner.SI_SetProperty(prop)
		Debug.Assert(result = SIResult.SIR_SUCCESS)

	End Sub

	Private Function InitializeBitmapFile() As BitmapFile

		Dim bmf As BitmapFile = New BitmapFile()

		Dim result As SIResult
		Dim prop As SIProperty = New SIProperty()

		' Get the current scan width (in pixels) from the scanner.
		prop.propertyID = SIP.SIP_SCAN_WIDTH_IN_PIXELS
		result = m_scanner.SI_GetProperty(prop)
		Debug.Assert(result = SIResult.SIR_SUCCESS)
		Debug.Assert((prop.containerType = SIP.SICON_RANGE) AndAlso (prop.itemType = SIP.SI_INT32))

		' Specify scan width to the file
		bmf.SetWidth(prop.range.current.iVal)

		' Get the current scan length (height) from the scanner.
		prop.propertyID = SIP.SIP_SCAN_LENGTH_IN_LINES
		result = m_scanner.SI_GetProperty(prop)
		Debug.Assert(result = SIResult.SIR_SUCCESS)
		Debug.Assert((prop.containerType = SIP.SICON_RANGE) AndAlso (prop.itemType = SIP.SI_INT32))

		' Specify scan length to the file. It will be vertically reversed, 
		' so negate the length.
		bmf.SetHeight(-prop.range.current.iVal)

		' Get the current number of bits per pixel from the scanner.
		prop.propertyID = SIP.SIP_BITS_PER_PIXEL
		result = m_scanner.SI_GetProperty(prop)
		Debug.Assert(result = SIResult.SIR_SUCCESS)
		Debug.Assert((prop.containerType = SIP.SICON_SINGLE) AndAlso (prop.itemType = SIP.SI_INT32))

		' Specify scan length to the file
		bmf.SetBitsPerPixel(prop.single.current.iVal)

		Return bmf

	End Function

	' returns the number of bytes in the buffer after alignment.
	Private Function AlignInPlace(ByRef buffer() As Byte, ByVal numberOfLines As Integer, ByVal bytesPerLine As Integer) As Integer

		If bytesPerLine Mod 4 Then

			Dim bytesPerLineAligned As Integer = bytesPerLine + ((4 - (bytesPerLine Mod 4)) Mod 4)
			Dim srcIndex As Integer = bytesPerLine * (numberOfLines - 1)
			Dim destIndex As Integer = bytesPerLineAligned * (numberOfLines - 1)

			While srcIndex >= 0

				Array.Copy(buffer, srcIndex, buffer, destIndex, bytesPerLine)

				destIndex -= bytesPerLineAligned
				srcIndex -= bytesPerLine

			End While

			Return bytesPerLineAligned * numberOfLines
		End If

		Return bytesPerLine * numberOfLines

	End Function

	Private Sub OnScanButtonClicked(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles scanButton.Click

		' Disable the scan button during scanning
		scanButton.Enabled = False

		' Check if calibration is needed and if so, do it.
		' This only returns false if there was an error.
		If Not CheckForCalibration() Then
			scanButton.Enabled = True
			Return
		End If

		' Make sure the user has paper inserted into the scanner.
		If Not ConfirmPaper("Please insert the item to be scanned and press OK." & vbCrLf & _
		 "Press Cancel to skip the scan.") Then

			scanButton.Enabled = True
			Return
		End If

		' Read the values from the UI controls and set them to the scanner.
		ControlsToProperties()

		' InitializeBitmapFile will read some of the default properties
		' from the scanner to initialize the bitmap file header.
		Dim file As BitmapFile = InitializeBitmapFile()

		' Open the file.
		Try

			file.Open(m_outFilename)

		Catch ee As Exception
			MessageBox.Show("Error opening the output file:  " & m_outFilename _
			 & vbCrLf & vbCrLf & ee.ToString(), "ScanToFile")
			scanButton.Enabled = True
			Return
		End Try


		' Allocate a buffer big enough for 10 lines.
		' We can find the space needed for one line by querying the
		' SIP_LINE_WIDTH_IN_BYTES property.
		Dim result As SIResult
		Dim prop As SIProperty = New SIProperty()

		prop.propertyID = SIP.SIP_LINE_WIDTH_IN_BYTES
		result = m_scanner.SI_GetProperty(prop)
		Debug.Assert(result = SIResult.SIR_SUCCESS)
		Debug.Assert((prop.containerType = SIP.SICON_SINGLE) AndAlso (prop.itemType = SIP.SI_INT32))

		Dim widthInBytes As Integer = prop.single.current.iVal

		' Allocate enough memory to hold the scan lines aligned on 32-bit boundaries.
		' Later we'll align all scan lines received to 32-bit boundaries because that's
		' what Windows Bitmaps require.
		Dim widthInBytesAligned As Integer = widthInBytes + ((4 - (widthInBytes Mod 4)) Mod 4)
		Dim lineBuffer((widthInBytesAligned * 10) - 1) As Byte			' Allocate the buffer

		' Now we're set to go.
		' Start the scan
		result = m_scanner.SI_StartScan()
		Debug.Assert(result = SIResult.SIR_SUCCESS)

		Dim numLinesReturned As UInteger
		Do

			numLinesReturned = 0

			' Attempt to read as many as 10 lines. We may get fewer than 
			' that if we're caught up to the scan.
			result = m_scanner.SI_ReadImageData(lineBuffer, 10, 0, numLinesReturned)

			' Align lines to 32-bit boundaries as required by Windows bitmap format 
			AlignInPlace(lineBuffer, numLinesReturned, widthInBytes)

			' Write the data out to the file.
			If result = SIResult.SIR_SUCCESS Then
				If numLinesReturned > 0 Then
					file.Write(lineBuffer, numLinesReturned * widthInBytesAligned)
				End If
			End If

		Loop While result = SIResult.SIR_SUCCESS

		If result = SIResult.SIR_ENDOFDATA Then

			' The full scan was done and we reached the end.

		ElseIf result <> SIResult.SIR_SUCCESS Then

			' error
			Dim errStr As String = ""
			m_scanner.SI_GetLastErrorText(errStr)
			MessageBox.Show(errStr, "ScanToFile")
		End If

		result = m_scanner.SI_EndScan()
		Debug.Assert(result = SIResult.SIR_SUCCESS)

		' Feed the paper the rest of the way out of the scanner.
		m_scanner.SI_FeedPaperOut()

		Try

			file.Close()

		Catch ee As Exception
			MessageBox.Show(ee.ToString(), "File Exception")
			scanButton.Enabled = True
			Return
		End Try

		' Launch the file we just scanned.
		Dim info As System.Diagnostics.ProcessStartInfo = New System.Diagnostics.ProcessStartInfo(m_outFilename)
		info.UseShellExecute = True
		info.Verb = "open"
		System.Diagnostics.Process.Start(info)

		scanButton.Enabled = True

	End Sub


	Protected Overrides Sub Finalize()
		MyBase.Finalize()
	End Sub


End Class

